
import requests_oauthlib
from oauthlib.oauth2 import TokenExpiredError
import time
import datetime
import base64
from settings import *
import requests


DEBUG = False

token = {'access_token': '',
         'scope': 'Read',
         'token_type': 'Bearer',
         'refresh_token': '',
         'expires_in': -10}



def get_latest_incidents(last_run_time):

    url = f"https://daas.api.hp.com/analytics/v1/incidents/1.2/tenants/{TENANT_ID}/incidents"

    payload = "{\n  \"tenantIds\": []\n}"
    headers = {
        'Content-Type': "application/json",
        # 'Authorization': f"Bearer {token['access_token']}",
    }
    params = {'filter': f'updatedAt ge "{last_run_time}"'}

    response = oauth.request("POST", url, params=params, data=payload, headers=headers)

    # print(response.text)

    incidents = response.json()['resources']

    for incident in incidents:
        if incident_wanted(incident):
            incident['weburl'] = f"https://www.hpdaas.com/ui/view/incidents/{incident['id']}"
            print(url)
            alert_in_teams(incident)

    # print(incidents)


def incident_wanted(incident):
    '''TODO Define logic in this function to filter for wanted events'''
    return True



def alert_in_teams(incident):

    payload = f"""{{
      "@context": "https://schema.org/extensions",
      "@type": "MessageCard",
      "themeColor": "0072C6",
      "title": "New HP TechPulse Incident: {incident['title']}",
      "text": "{incident['title']} - Click **View** to view the incident in HP TechPulse !",
      "potentialAction": [
        {{
          "@type": "OpenUri",
          "name": "View",
          "targets": [
            {{ "os": "default", "uri": "{incident['weburl']}" }}
          ]
        }}
      ]
        }}"""

    print(payload)

    headers = {
        'Content-Type': 'application/json'
    }

    response = requests.request("POST", TEAMS_WEBHOOK_URL, headers=headers, data=payload)

    print(response.text.encode('utf8'))






oauth = requests_oauthlib.OAuth2Session(CLIENT_ID, redirect_uri=REDIRECT_URI, scope=SCOPE, token=token)
authorization_url, state = oauth.authorization_url(AUTHORIZE_URL)

print(f'Please go to {authorization_url} and authorize access.')
authorization_response = input('Enter the full callback URL: ')

token = oauth.fetch_token('https://daas.api.hp.com/oauth/v1/token',
                          authorization_response=authorization_response,
                          client_id=CLIENT_ID,
                          client_secret=CLIENT_SECRET,
                          include_client_id=True)

max_attempts = 15

while True:

    if DEBUG:
        print(token)

    now = datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S')  # Example: 2019-03-05 00:00:00

    # setup last_run_time
    if 'last_run_time' in locals():
        pass
    else:
        last_run_time = now

    for attempt in range(max_attempts):

        try:
            get_latest_incidents(last_run_time)
            attempt_good = True

        except TokenExpiredError as e:
            print('trying to refresh token')

            # concatenate with : and encode in base64 (basic auth) and turn bytes object into ascii string
            b64_clientid_clientsecret = base64.b64encode(str.encode(f'{CLIENT_ID}:{CLIENT_SECRET}')).decode('ascii')
            headers = {
                'Content-Type': "application/x-www-form-urlencoded",
                'Authorization': f'Basic {b64_clientid_clientsecret}'
            }
            token = oauth.refresh_token(REFRESH_URL, refresh_token=token['refresh_token'], headers=headers)
            attempt_good = False

        if attempt_good:
            last_run_time = now
            break

    sleep_time = 60
    print(f'Sleeping for {sleep_time}')
    time.sleep(sleep_time)



